/*
 * Copyright 2024 Velocity Team
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

const { flow, get, isEmpty, omit, set } = require('lodash/fp');
const {
  encryptCollection,
  decryptCollection,
} = require('@velocitycareerlabs/crypto');

const initEncryptPropExtension = ({
  prop,
  encryptedProp,
  secret,
  format = 'string',
} = {}) => {
  const encryptProp = (obj) => {
    const value = get(prop, obj);
    if (isEmpty(value)) {
      return obj;
    }

    const serializedValue = format === 'json' ? JSON.stringify(value) : value;
    const encryptedValue = encryptCollection(serializedValue, secret);
    return flow(omit([prop]), set(encryptedProp, encryptedValue))(obj);
  };

  const decryptProp = (obj) => {
    const value = get(encryptedProp, obj);
    if (isEmpty(value)) {
      return obj;
    }

    const decryptedValue = decryptCollection(value, secret);
    const unserializedValue =
      format === 'json' ? JSON.parse(decryptedValue) : decryptedValue;
    return set(prop, unserializedValue, obj);
  };

  return (parent) => ({
    prepModification: (rawVal, ...args) => {
      const val = encryptProp(rawVal);
      return parent.prepModification(val, ...args);
    },
    findOneAndDecrypt: async (...args) => {
      const val =
        parent.findOneAndDecrypt != null
          ? await parent.findOneAndDecrypt(...args)
          : await parent.findOne(...args);
      return decryptProp(val);
    },
    decryptProp,
  });
};

module.exports = {
  initEncryptPropExtension,
};
